@@ -44,4 +44,19 @@ bool render_page(render_thread_t* render_thread, zathura_page_t* page);
  */
 void render_all(zathura_t* zathura);
 
+/**
+ * Lock the render thread. This is useful if you want to render on your own (e.g
+ * for printing).
+ *
+ * @param render_thread The render thread object.
+ */
+void render_lock(render_thread_t* render_thread);
+
+/**
+ * Unlock the render thread.
+ *
+ * @param render_thread The render thread object.
+ */
+void render_unlock(render_thread_t* render_thread);
+
 #endif // RENDER_H
